#!/bin/bash

################################################################################
#
#    Skript pro generovani napovedy projektu CoronaExt
#
#    (C) 2011 Simopt, s.r.o.   
#
################################################################################

#-------------------------------------------------------------------------------
# Konfigurace skriptu
#-------------------------------------------------------------------------------

# Root adresar dokumentace
DOC_ROOT_FOLDER="html";

# Vychozi adresar pro prohledavani dokumentace
SRC_ROOT_FOLDER="../.."

#-------------------------------------------------------------------------------
# Generuje hlavicku HTML stranky
# $1 title
#-------------------------------------------------------------------------------

function generateHTMLHeader {

    echo "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">"
    echo "<html xmlns=\"http://www.w3.org/1999/xhtml\" lang=\"en\" xml:lang=\"en\">"
	echo "<head>"
	echo "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\"/>"
	echo "<title>$1</title>"
	echo "</head>"

}

#-------------------------------------------------------------------------------
# Generuje paticku HTML stranky
#-------------------------------------------------------------------------------

function generateHTMLFooter {

	echo "</html>"

}

#-------------------------------------------------------------------------------
# Generuje hlavicku list.html - seznamu trid
#-------------------------------------------------------------------------------

function generateListHTMLHeader {

    list_file="${DOC_ROOT_FOLDER}/list.html"
    
    echo "" > "$list_file"
    
    generateHTMLHeader "Třídy" >> "$list_file"
    
    echo "<body>" >> "$list_file"
    echo "<h1 style=\"text-align: center;\">Třídy</h1>" >> "$list_file"
    echo "<ul style=\"list-style-type: none; line-height: 1.5;\">" >> "$list_file"    
}

#-------------------------------------------------------------------------------
# Generuje paticku list.html - seznamu trid
#-------------------------------------------------------------------------------

function generateListHTMLFooter {

    list_file="${DOC_ROOT_FOLDER}/list.html"
        
    echo "</ul>" >> "$list_file"
    echo "</body>" >> "$list_file"
    
    generateHTMLFooter >> "$list_file"

}

#-------------------------------------------------------------------------------
# Generuje polozku seznamu do list.html s tridama
# $1 html_link
# $2 class_name
#-------------------------------------------------------------------------------

function generateListHTMLRecord {

    list_file="${DOC_ROOT_FOLDER}/list.html"

    echo "<li><a href=\"$1\" target=\"main\">$1</a></li>" >> "$list_file"

}

#-------------------------------------------------------------------------------
# Generuje index.html
#-------------------------------------------------------------------------------

function generateIndexHTML {

    index_file="${DOC_ROOT_FOLDER}/index.html";
    
    echo "" > $index_file

    generateHTMLHeader "Dokumentace" >> $index_file

    echo "<frameset cols=\"400, *\">" >> $index_file
    echo "<frame name=\"list\" src=\"list.html\">" >> $index_file
    echo "<frame name=\"main\" src=\"list.html\">" >> $index_file
    echo "</frameset>" >> $index_file
    
    generateHTMLFooter >> $index_file

}

#-------------------------------------------------------------------------------
# Dany radek upravi tak, ze vyhleda vyskyt odkazu na CoronaExt tridu a aktivuje
# jej - udela z nej HTML link
# $1 line
# $2 class_name
#-------------------------------------------------------------------------------

function generateCoronaExtClassLink {

	line="$1"
	class_name="$2"	
	
	link_class=`echo $line | sed 's/.*(\(coronaext\/.*\)).*/\1/'`
	
	if [ "$link_class" == "$line" ]; then
		echo "$line"
		return
	fi
	
	link_dirname=`dirname "$link_class"`
	link_basename=`basename "$link_class"`	
	
	class_dirname=`dirname "$class_name"`
	class_basename=`basename "$class_name"`	

	final_link="";
	help_path="";

	p=0
	total_class=0;
	for i in $(echo $class_dirname | tr "/" "\n"); do 
		class_array[$p]="$i"
		p=$(($p + 1))
		total_class="$p"
	done

	p=0
	for i in $(echo $link_dirname | tr "/" "\n"); do 
		link_array[$p]="$i"	
		
		if [ ! ${class_array[$p]} == "" ]; then
			
			if [ ! "${class_array[$p]}" == "$i" ]; then
				
				final_link="../$final_link"
				
				if [ "$help_path" == "" ]; then
					help_path="$i"
				else
					help_path="$help_path/$i"
				fi
				
			fi
			
		else
			
			if [ "$help_path" == "" ]; then
				help_path="$i"
			else
				help_path="$help_path/$i"
			fi
			
		fi
		
		p=$(($p + 1))
	done
	
	if [ "$total_class" -gt "$p" ]; then
		diff=$(($total_class - $p))		
		for i in `seq 1 $diff`; do
			final_link="../$final_link"
		done
	fi	
	
	href_link="${final_link}${help_path}/${link_basename}.html"
	if [ "$help_path" == "" ]; then
		href_link="${final_link}${link_basename}.html"
	fi
	
	new_line=`echo $line | sed 's/(\(coronaext\/.*\))/(<a href="XXX_HREF_LINK_XXX" target="main">\1<\/a>)/'`
		
	str_to_replace="XXX_HREF_LINK_XXX"
	new_line=${new_line/${str_to_replace}/${href_link}}
		
	echo "$new_line"
	
}

#-------------------------------------------------------------------------------
# Generuje dokumentaci z daneho zdrojoveho souboru
# $1 src_file
# $2 class_name
#-------------------------------------------------------------------------------

function generateDocFromSourceFile {

    src_file="$1"
    class_name="$2"
    
    start_doc_regexp="^.*<doc>.*$"
    end_doc_regexp="^.*</doc>.*$"
    comment_line_regexp="^\s*---.*$"
    function_line_regexp="^\s*function.*$"
    
    in_doc="false"
    
    while read line; do

        set -- $line;
        
        if [[ "$in_doc" == "false" ]]; then        
            
            if [[ $line =~ $start_doc_regexp ]]; then
                in_doc="true";            
            fi
            
        else # if in_doc == true
        
            if [[ $line =~ $end_doc_regexp ]]; then
                in_doc="false";            
            else

                # generuje HTML linky
                line=`generateCoronaExtClassLink "$line" "$class_name"`
                
                # vypis radku podle typu radku
                if [[ "$line" =~ $comment_line_regexp ]]; then
                    echo "<span style=\"color:#5379C7;\">$line</span>"
                else
                    
                    if  [[ "$line" =~ $function_line_regexp ]]; then
                        echo "<span style=\"color:#AA0000;\">$line</span>"
                    else
                        echo "$line"
                    fi
                    
                fi
                
            fi
        
        fi
                
    done <"$src_file"
    
}

#-------------------------------------------------------------------------------
# generuje HTML dokumentaci z lua souboru
# $1 file_name
# $2 src_file
# $3 html_file
# $4 html_link
# $5 class_name
# $6 html_dir
#-------------------------------------------------------------------------------

function generateHTMLDocFile {
    
    file_name="$1"
    src_file="$2"
    html_file="$3"
    html_link="$4"
    class_name="$5"
    html_dir="$6"

    mkdir -p "$html_dir"
    
    echo "" > "$html_file"
    generateHTMLHeader "$class_name" >> "$html_file"
    echo "<body>" >> "$html_file"    
    echo "<h1 style=\"color:#5379C7;\">$class_name</h1>" >> "$html_file"
    echo "<pre>" >> "$html_file"    

    # generujeme obsah souboru ...
    generateDocFromSourceFile "$src_file" "$class_name" >> "$html_file"
    
    
    echo "</pre>" >> "$html_file"
    echo "</body>" >> "$html_file"
    generateHTMLFooter >> "$html_file"

}

#-------------------------------------------------------------------------------
# Vyhleda zdrojove soubory
#-------------------------------------------------------------------------------

function processLuaFiles {

    files=`find "$SRC_ROOT_FOLDER" -type f`;
    for src_file in $files; do
                                    
        # nazev repozitare
        file_name=`basename "$src_file"`;
        
        # vysledny html soubor
        html_file=${src_file/$SRC_ROOT_FOLDER/${DOC_ROOT_FOLDER}/coronaext}
        html_file=${html_file/lua/html}
        
        # vysledny html link
        html_link=${html_file/html\//}
        
        # adresar ve kterem bude html soubor
        html_dir=`dirname "$html_file"`;
        
        # nazev tridy
        class_name=${html_link/\.html/}
        
        # regularni vyraz pro zjisteni, zda-li je dany adresar repozitar
        src_regexp="^..*\.lua$";

        # test, zda-li adresar nezacina podrtzitkem - tyto adresare vynechavame
        help_dir_regexp="_.*";

        if [[ "$html_dir" =~ $help_dir_regexp ]]; then

            echo "Vynechavam adresar: $html_dir"

        else

            # test repozitare
            if [[ $file_name =~ $src_regexp ]]; then

                echo "file_name = $file_name";
                echo "src_file = $src_file";
                echo "html_file = $html_file";
                echo "html_link = $html_link";
                echo "class_name = $class_name";
                echo "html_dir = $html_dir";
                echo "";

                generateHTMLDocFile "$file_name" "$src_file" "$html_file" "$html_link" "$class_name" "$html_dir"
                generateListHTMLRecord "$html_link" "$class_name"

            fi

        fi
        
    done

}

#-------------------------------------------------------------------------------
# Vycisteni stare dokumentace
#-------------------------------------------------------------------------------

function removeOldHTMLDoc {

    rm -fr "$DOC_ROOT_FOLDER"

}

#-------------------------------------------------------------------------------
# spusteni skriptu
#-------------------------------------------------------------------------------

removeOldHTMLDoc

mkdir -p "$DOC_ROOT_FOLDER"

generateIndexHTML

generateListHTMLHeader

processLuaFiles

generateListHTMLFooter
